@@ -7,15 +7,67 @@ from zmq.utils import jsonapi
 from tornado.queues import Queue
 from tornado.locks import Event
 
-from .compiler import (get_file_name, get_tmp_directory, get_tmp_hash_seed)
-
 from IPython.core.getipython import get_ipython
 
-from .jsonutil import json_clean
+try:
+    from jupyter_client.jsonutil import json_default
+except ImportError:
+    from jupyter_client.jsonutil import date_default as json_default
+
+from .compiler import (get_file_name, get_tmp_directory, get_tmp_hash_seed)
+
+# This import is required to have the next ones working...
+from debugpy.server import api  # noqa
+from _pydevd_bundle import pydevd_frame_utils
+from _pydevd_bundle.pydevd_suspended_frames import SuspendedFramesManager, _FramesTracker
 
 # Required for backwards compatiblity
 ROUTING_ID = getattr(zmq, 'ROUTING_ID', None) or zmq.IDENTITY
 
+
+class _FakeCode:
+    def __init__(self, co_filename, co_name):
+        self.co_filename = co_filename
+        self.co_name = co_name
+
+
+class _FakeFrame:
+    def __init__(self, f_code, f_globals, f_locals):
+        self.f_code = f_code
+        self.f_globals = f_globals
+        self.f_locals = f_locals
+        self.f_back = None
+
+
+class _DummyPyDB:
+    def __init__(self):
+        from _pydevd_bundle.pydevd_api import PyDevdAPI
+        self.variable_presentation = PyDevdAPI.VariablePresentation()
+
+
+class VariableExplorer:
+    def __init__(self):
+        self.suspended_frame_manager = SuspendedFramesManager()
+        self.py_db = _DummyPyDB()
+        self.tracker = _FramesTracker(self.suspended_frame_manager, self.py_db)
+        self.frame = None
+
+    def track(self):
+        var = get_ipython().user_ns
+        self.frame = _FakeFrame(_FakeCode('<module>', get_file_name('sys._getframe()')), var, var)
+        self.tracker.track('thread1', pydevd_frame_utils.create_frames_list_from_frame(self.frame))
+
+    def untrack_all(self):
+        self.tracker.untrack_all()
+
+    def get_children_variables(self, variable_ref = None):
+        var_ref = variable_ref
+        if not var_ref:
+            var_ref = id(self.frame)
+        variables = self.suspended_frame_manager.get_variable(var_ref)
+        return [x.get_var_data() for x in variables.get_children_variables()]
+
+
 class DebugpyMessageQueue:
 
     HEADER = 'Content-Length: '
@@ -58,7 +110,7 @@ class DebugpyMessageQueue:
                 self.header_pos = self.tcp_buffer.find(DebugpyMessageQueue.HEADER)
             if self.header_pos == -1:
                 return
-        
+
             self.log.debug('QUEUE - found header at pos %i', self.header_pos)
 
             #Finds separator
@@ -94,7 +146,7 @@ class DebugpyMessageQueue:
 
     async def get_message(self):
         return await self.message_queue.get()
-            
+
 
 class DebugpyClient:
 
@@ -123,7 +175,12 @@ class DebugpyClient:
     def _send_request(self, msg):
         if self.routing_id is None:
             self.routing_id = self.debugpy_stream.socket.getsockopt(ROUTING_ID)
-        content = jsonapi.dumps(msg)
+        content = jsonapi.dumps(
+            msg,
+            default=json_default,
+            ensure_ascii=False,
+            allow_nan=False,
+        )
         content_length = str(len(content))
         buf = (DebugpyMessageQueue.HEADER + content_length + DebugpyMessageQueue.SEPARATOR).encode('ascii')
         buf += content
@@ -131,7 +188,7 @@ class DebugpyClient:
         self.log.debug(self.routing_id)
         self.log.debug(buf)
         self.debugpy_stream.send_multipart((self.routing_id, buf))
-    
+
     async def _wait_for_response(self):
         # Since events are never pushed to the message_queue
         # we can safely assume the next message in queue
@@ -141,7 +198,7 @@ class DebugpyClient:
     async def _handle_init_sequence(self):
         # 1] Waits for initialized event
         await self.init_event.wait()
-        
+
         # 2] Sends configurationDone request
         configurationDone = {
             'type': 'request',
@@ -193,6 +250,7 @@ class DebugpyClient:
             self.log.debug(rep)
             return rep
 
+
 class Debugger:
 
     # Requests that requires that the debugger has started
@@ -202,7 +260,7 @@ class Debugger:
         'variables', 'attach',
         'configurationDone'
     ]
-    
+
     # Requests that can be handled even if the debugger is not running
     static_debug_msg_types = [
         'debugInfo', 'inspectVariables', 'richInspectVariables'
@@ -215,7 +273,7 @@ class Debugger:
         self.session = session
         self.is_started = False
         self.event_callback = event_callback
-        
+
         self.started_debug_handlers = {}
         for msg_type in Debugger.started_debug_msg_types:
             self.started_debug_handlers[msg_type] = getattr(self, msg_type)
@@ -233,6 +291,8 @@ class Debugger:
         self.debugpy_port = 0
         self.endpoint = None
 
+        self.variable_explorer = VariableExplorer()
+
     def _handle_event(self, msg):
         if msg['event'] == 'stopped':
             self.stopped_threads.append(msg['body']['threadId'])
@@ -246,6 +306,20 @@ class Debugger:
     async def _forward_message(self, msg):
         return await self.debugpy_client.send_dap_request(msg)
 
+    def _build_variables_response(self, request, variables):
+        var_list = [var for var in variables if self.accept_variable(var['name'])]
+        reply = {
+            'seq': request['seq'],
+            'type': 'response',
+            'request_seq': request['seq'],
+            'success': True,
+            'command': request['command'],
+            'body': {
+                'variables': var_list
+            }
+        }
+        return reply
+
     @property
     def tcp_client(self):
         return self.debugpy_client
@@ -264,7 +338,7 @@ class Debugger:
             }
             self.session.send(self.shell_socket, 'execute_request', content,
                               None, (self.shell_socket.getsockopt(ROUTING_ID)))
-                          
+
             ident, msg = self.session.recv(self.shell_socket, mode=0)
             self.debugpy_initialized = msg['content']['status'] == 'ok'
         self.debugpy_client.connect_tcp_socket()
@@ -370,10 +444,15 @@ class Debugger:
         return cond
 
     async def variables(self, message):
-        reply = await self._forward_message(message)
-        # TODO : check start and count arguments work as expected in debugpy
-        reply['body']['variables'] = \
-            [var for var in reply['body']['variables'] if self.accept_variable(var['name'])]
+        reply = {}
+        if not self.stopped_threads:
+            variables = self.variable_explorer.get_children_variables(message['arguments']['variablesReference'])
+            return self._build_variables_response(message, variables)
+        else:
+            reply = await self._forward_message(message)
+            # TODO : check start and count arguments work as expected in debugpy
+            reply['body']['variables'] = \
+                [var for var in reply['body']['variables'] if self.accept_variable(var['name'])]
         return reply
 
     async def attach(self, message):
@@ -414,39 +493,41 @@ class Debugger:
                 'tmpFilePrefix': get_tmp_directory() + '/',
                 'tmpFileSuffix': '.py',
                 'breakpoints': breakpoint_list,
-                'stoppedThreads': self.stopped_threads
+                'stoppedThreads': self.stopped_threads,
+                'richRendering': True
             }
         }
         return reply
 
     async def inspectVariables(self, message):
-        var_list = []
-        for k, v in get_ipython().user_ns.items():
-            if self.accept_variable(k):
-                try:
-                    val = json_clean(v)
-
-                except ValueError:
-                    val = str(v)
-                var_list.append({
-                    'name': k,
-                    'value': val,
-                    'type': str(type(v))[8:-2],
-                    'variablesReference': 0
-                })
-        reply = {
-            'type': 'response',
-            'request_seq': message['seq'],
-            'success': True,
-            'command': message['command'],
-            'body': {
-                'variables': var_list
-            }
-        }
-        return reply
+        self.variable_explorer.untrack_all()
+        # looks like the implementation of untrack_all in ptvsd
+        # destroys objects we nee din track. We have no choice but
+        # reinstantiate the object
+        self.variable_explorer = VariableExplorer()
+        self.variable_explorer.track()
+        variables = self.variable_explorer.get_children_variables()
+        return self._build_variables_response(message, variables)
 
     async def richInspectVariables(self, message):
+        reply = {
+            'type': 'response',
+            'sequence_seq': message['seq'],
+            'success': False,
+            'command': message['command']
+        }
+        
         var_name = message['arguments']['variableName']
+        valid_name = str.isidentifier(var_name)
+        if not valid_name:
+            reply['body'] = {
+                'data': {},
+                'metadata': {}
+            }
+            if var_name == 'special variables' or var_name == 'function variables':
+                reply['success'] = True
+            return reply
+
         var_repr_data = var_name + '_repr_data'
         var_repr_metadata = var_name + '_repr_metadata'
 
@@ -468,28 +549,21 @@ class Debugger:
                 'arguments': {
                     'expression': lvalue,
                     'value': code,
-                    'frameId': frameId
+                    'frameId': frame_id
                 }
             }
             await self._forward_message(request)
 
-        reply = {
-            'type': 'response',
-            'sequence_seq': message['seq'],
-            'success': False,
-            'command': message['command']
-        }
-
         repr_data = globals()[var_repr_data]
         repr_metadata = globals()[var_repr_metadata]
         body = {
             'data': {},
             'metadata': {}
         }
-        
+
         for key, value in repr_data.items():
             body['data']['key'] = value
-            if repr_metadata.has_key(key):
+            if key in repr_metadata:
                 body['metadata'][key] = repr_metadata[key]
 
         globals().pop(var_repr_data)
