@@ -10,6 +10,7 @@ import errno
 import signal
 import traceback
 import logging
+from functools import partial
 from io import TextIOWrapper, FileIO
 from logging import StreamHandler
 
@@ -129,7 +130,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp,
     iopub_thread = Any()
     control_thread = Any()
 
-    ports = Dict()
+    _ports = Dict()
 
     subcommands = {
         'install': (
@@ -162,6 +163,12 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp,
     displayhook_class = DottedObjectName('ipykernel.displayhook.ZMQDisplayHook',
         help="The importstring for the DisplayHook factory").tag(config=True)
 
+    capture_fd_output = Bool(
+        True,
+        help="""Attempt to capture and forward low-level output, e.g. produced by Extension libraries.
+    """,
+    ).tag(config=True)
+
     # polling
     parent_handle = Integer(int(os.environ.get('JPY_PARENT_PID') or 0),
         help="""kill this process if its parent dies.  On Windows, the argument
@@ -390,7 +397,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp,
             for line in lines:
                 print(line, file=sys.__stdout__)
 
-        self.ports = dict(shell=self.shell_port, iopub=self.iopub_port,
+        self._ports = dict(shell=self.shell_port, iopub=self.iopub_port,
                                 stdin=self.stdin_port, hb=self.hb_port,
                                 control=self.control_port)
 
@@ -413,6 +420,9 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp,
             e_stdout = None if self.quiet else sys.__stdout__
             e_stderr = None if self.quiet else sys.__stderr__
 
+            if not self.capture_fd_output:
+                outstream_factory = partial(outstream_factory, watchfd=False)
+
             sys.stdout = outstream_factory(self.session, self.iopub_thread,
                                            'stdout',
                                            echo=e_stdout)
@@ -500,7 +510,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp,
                                 user_ns=self.user_ns,
         )
         kernel.record_ports({
-            name + '_port': port for name, port in self.ports.items()
+            name + '_port': port for name, port in self._ports.items()
         })
         self.kernel = kernel
 
